!***************************************************************************
! September 21 2000 version
! checkoutput - tests omflatgen reads in the 4 images that are stored 
!               in the file INDEX.FIT and then checks that the flatfield
!               that has been created by it has been normalised correctly.
!               It checks this by comparing the counts in the created 
!               flatfield file with those in the file consisting of the four
!               separate files after they have been normalized.
!**************************************************************************                

program checkoutput

    use dal
    use errorhandling

    implicit none

    
    real(kind=single), dimension(0 : 1023, 0 : 1023) :: testFlat
  
    type(DataSetT) taskDataSetH
    type(ArrayT) taskArrayH
    real(kind=single), dimension(:, :), pointer :: taskFlat

    real counter
    integer nDim
    integer i, j, k, m
    

! Create the original image from the 4 separate images that go into omflatgen
    call CreateImage(testFlat) 



! Normalise the test flat image  data
    do i = 1, 4 ! Split the image up into 4 segments
        counter = 0
        m = (i - 1) * 256
        do j = 0, 255
            do k = 0, 1023
                counter = counter + testFlat(m + j, k)
            end do
        end do
        counter = counter / 262144.0
        do j = 0, 255
            do k = 0, 1023
                testFlat(m + j, k) = testFlat(m + j, k) / counter
            end do
        end do
    end do
    
! Read in the task field

    taskDataSetH = dataSet("flatfield.fit", read)
    taskArrayH = array(taskDataSetH, "PRIMARY", read)
    taskFlat => real32Array2Data(taskArrayH)

    ndim = numberofDimensions(taskArrayH)

    if (nDim /= 2) then
        call error("Not image", "Data in PRIMARY array is not an image")
    end if

! Check against test

    do i = 0, 1023
        do j = 0, 1023
            if (taskFlat(i, j) /= testFlat(i, j)) then
                call fatal("Failed", "Values in task flat are incorrect")
            end if            
        end do
    end do
     
! Release handles

    call release(taskArrayH)
    call release(taskDataSetH)
    stop  

contains
!********************************************************
!
!
!*******************************************************
subroutine CreateImage(data)
    real(kind=single), dimension(0 : 1023, 0 : 1023), intent(inout) :: data
   
    call AddDataFile("LED0100M8.FIT", 1, data)
    call AddDataFile("LED0100M8.FIT", 2, data)
    call AddDataFile("LED0100M8.FIT", 3, data)
    call AddDataFile("LED0100M8.FIT", 4, data)
end subroutine CreateImage
!**********************************************************
!
!
!**********************************************************
subroutine AddDataFile(fileName, index, data)
    character(len=*), intent(in) :: fileName
    integer, intent(in) :: index
    real(kind=single), dimension(0 : 1023, 0 : 1023), intent(inout) :: data
    integer i, j, k
    type(DataSetT) dataSetH
    
    type(ArrayT)  arrayH
    real(kind=single), dimension(:, :),  pointer :: dataArray
  
    dataSetH = dataSet(fileName, read)
    arrayH = array(dataSetH, "PRIMARY", read)
    dataArray => real32Array2Data(arrayH)

    k = (index - 1) * 256
    do i = 0, 255
        do j = 0, 255
            data(k + i, j) = dataArray(i, j)
        end do
    end do
    call release(arrayH)
    call release(dataSetH)
end subroutine AddDataFile
 
end program checkoutput








